package com.ocity.api.auth;

import com.ocity.api.util.JwtUtil;
import io.jsonwebtoken.Claims;
import jakarta.annotation.Nonnull;
import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.util.AntPathMatcher;
import org.springframework.web.filter.OncePerRequestFilter;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Component
public class JwtAuthFilter extends OncePerRequestFilter {
    private static final List<String> WHITE_LIST_URL = Arrays.asList(
            "/login",
            "/**.html",
            "/js/**.js",
            "/css/**.css",
            "/favicon.ico",
            "/mock" //TODO remove
    );

    private final JwtUtil jwtUtil;
    private final ApiKeyRetriever apiKeyRetriever;

    @Autowired
    public JwtAuthFilter(JwtUtil jwtUtil, ApiKeyRetriever apiKeyRetriever) {
        this.jwtUtil = jwtUtil;
        this.apiKeyRetriever = apiKeyRetriever;
    }

    @Override
    public void doFilterInternal(@Nonnull HttpServletRequest request, @Nonnull HttpServletResponse response, @Nonnull FilterChain filterChain) throws ServletException, IOException {
        final Optional<String> apiKey = apiKeyRetriever.execute(request);

        if (apiKey.isEmpty() || !jwtUtil.validateToken(apiKey.get())) {
            SecurityContextHolder.clearContext();
            filterChain.doFilter(request, response);
            return;
        }

        String apiKeyValue = apiKey.get();
        Claims claims = jwtUtil.getAllClaimsFromToken(apiKeyValue);
        List<String> roles = claims.get("roles", List.class);

        List<GrantedAuthority> authorities = roles.stream()
                .map(SimpleGrantedAuthority::new)
                .collect(Collectors.toList());

        SecurityContextHolder.getContext().setAuthentication(new ApiKeyAuthenticationToken(apiKeyValue, authorities));

        filterChain.doFilter(request, response);
    }

    @Override
    protected boolean shouldNotFilter(HttpServletRequest request) {
        return WHITE_LIST_URL.stream()
                .anyMatch(url -> new AntPathMatcher().match(url, request.getServletPath()));
    }
}
