package com.ocity.api.core.infrastructure.heritage;

import com.ocity.api.core.application.heritage.dto.in.HeritageFormat;
import com.ocity.api.core.domain.heritage.HeritageItem;
import com.ocity.api.core.domain.heritage.HeritageRepository;
import com.ocity.api.core.domain.heritage.SubregionId;
import com.ocity.api.core.domain.heritage.Type;
import com.ocity.api.core.domain.heritage.Types;
import com.ocity.api.core.domain.locations.CityId;
import com.ocity.api.core.domain.locations.CountryId;
import com.ocity.api.core.domain.locations.RegionId;
import com.ocity.api.core.domain.locations.StateId;
import com.ocity.api.core.infrastructure.heritage.entity.FieldTagsEntity;
import com.ocity.api.core.infrastructure.heritage.entity.HeritageItemEntity;
import com.ocity.api.core.infrastructure.heritage.entity.HeritageItemMapper;
import com.ocity.api.core.infrastructure.heritage.entity.HeritageListLinksEntity;
import com.ocity.api.core.infrastructure.locations.entity.CountryEntity;
import jakarta.persistence.EntityManager;
import jakarta.persistence.PersistenceContext;
import org.springframework.stereotype.Repository;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Repository
public class HeritageRepositoryAdapter implements HeritageRepository {

    private static final String CATEGORIES_PARAM = "categories";

    private final HeritageItemMapper heritageItemMapper;

    @PersistenceContext
    EntityManager entityManager;

    public HeritageRepositoryAdapter(HeritageItemMapper heritageItemMapper) {
        this.heritageItemMapper = heritageItemMapper;
    }

    @Override
    public List<HeritageItem> findByRegionId(RegionId regionId, Types types, HeritageFormat heritageFormat) {
        List<Integer> countryIds = entityManager
                .createNamedQuery(CountryEntity.FIND_COUNTRIES_IDS_BY_REGION_ID, Integer.class)
                .setParameter("regionId", regionId.value())
                .getResultList();

        List<HeritageItemEntity> heritageItemEntityList = entityManager
                .createNamedQuery(HeritageItemEntity.FIND_BY_REGION_ID_AND_CATEGORIES, HeritageItemEntity.class)
                .setParameter("countryIds", countryIds)
                .setParameter(CATEGORIES_PARAM, mapCategories(types))
                .getResultList();

        Map<Integer, List<HeritageListLinksEntity>> heritageListLinksByHeritageListId = findHeritageListLinksByHeritageListId(heritageItemEntityList);
        Map<Integer, List<FieldTagsEntity>> tagsByHeritageListId = findTagsByHeritageListId(heritageItemEntityList);

        return heritageItemEntityList.stream()
                .map(heritageItemEntity -> heritageItemMapper.map(heritageItemEntity, heritageListLinksByHeritageListId, tagsByHeritageListId, heritageFormat))
                .toList();
    }

    @Override
    public List<HeritageItem> findBySubregionId(SubregionId subregionId, Types types, HeritageFormat heritageFormat) {
        List<Integer> countryIds = entityManager
                .createNamedQuery(CountryEntity.FIND_COUNTRIES_IDS_BY_SUBREGION_ID, Integer.class)
                .setParameter("subregionId", subregionId.value())
                .getResultList();

        List<HeritageItemEntity> heritageItemEntityList = entityManager
                .createNamedQuery(HeritageItemEntity.FIND_BY_SUBREGION_ID_AND_CATEGORIES, HeritageItemEntity.class)
                .setParameter("countryIds", countryIds)
                .setParameter(CATEGORIES_PARAM, mapCategories(types))
                .getResultList();

        Map<Integer, List<HeritageListLinksEntity>> heritageListLinksByHeritageListId = findHeritageListLinksByHeritageListId(heritageItemEntityList);
        Map<Integer, List<FieldTagsEntity>> tagsByHeritageListId = findTagsByHeritageListId(heritageItemEntityList);

        return heritageItemEntityList.stream()
                .map(heritageItemEntity -> heritageItemMapper.map(heritageItemEntity, heritageListLinksByHeritageListId, tagsByHeritageListId, heritageFormat))
                .toList();
    }

    @Override
    public List<HeritageItem> findByCountryId(CountryId countryId, Types types, HeritageFormat heritageFormat) {
        List<HeritageItemEntity> heritageItemEntityList = entityManager
                .createNamedQuery(HeritageItemEntity.FIND_BY_COUNTRY_ID_AND_CATEGORIES, HeritageItemEntity.class)
                .setParameter("countryId", countryId.value())
                .setParameter(CATEGORIES_PARAM, mapCategories(types))
                .getResultList();

        Map<Integer, List<HeritageListLinksEntity>> heritageListLinksByHeritageListId = findHeritageListLinksByHeritageListId(heritageItemEntityList);
        Map<Integer, List<FieldTagsEntity>> tagsByHeritageListId = findTagsByHeritageListId(heritageItemEntityList);

        return heritageItemEntityList.stream()
                .map(heritageItemEntity -> heritageItemMapper.map(heritageItemEntity, heritageListLinksByHeritageListId, tagsByHeritageListId, heritageFormat))
                .toList();
    }

    @Override
    public List<HeritageItem> findByStateId(StateId stateId, Types types, HeritageFormat heritageFormat) {
        List<HeritageItemEntity> heritageItemEntityList = entityManager
                .createNamedQuery(HeritageItemEntity.FIND_BY_STATE_ID_AND_CATEGORIES, HeritageItemEntity.class)
                .setParameter("stateId", stateId.value())
                .setParameter(CATEGORIES_PARAM, mapCategories(types))
                .getResultList();

        Map<Integer, List<HeritageListLinksEntity>> heritageListLinksByHeritageListId = findHeritageListLinksByHeritageListId(heritageItemEntityList);
        Map<Integer, List<FieldTagsEntity>> tagsByHeritageListId = findTagsByHeritageListId(heritageItemEntityList);

        return heritageItemEntityList.stream()
                .map(heritageItemEntity -> heritageItemMapper.map(heritageItemEntity, heritageListLinksByHeritageListId, tagsByHeritageListId, heritageFormat))
                .toList();
    }

    @Override
    public List<HeritageItem> findByCityId(CityId cityId, Types types, HeritageFormat heritageFormat) {
        List<HeritageItemEntity> heritageItemEntityList = entityManager
                .createNamedQuery(HeritageItemEntity.FIND_BY_CITY_ID_AND_CATEGORIES, HeritageItemEntity.class)
                .setParameter("cityId", cityId.value())
                .setParameter(CATEGORIES_PARAM, mapCategories(types))
                .getResultList();

        Map<Integer, List<HeritageListLinksEntity>> heritageListLinksByHeritageListId = findHeritageListLinksByHeritageListId(heritageItemEntityList);
        Map<Integer, List<FieldTagsEntity>> tagsByHeritageListId = findTagsByHeritageListId(heritageItemEntityList);

        return heritageItemEntityList.stream()
                .map(heritageItemEntity -> heritageItemMapper.map(heritageItemEntity, heritageListLinksByHeritageListId, tagsByHeritageListId, heritageFormat))
                .toList();
    }

    private Map<Integer, List<FieldTagsEntity>> findTagsByHeritageListId(List<HeritageItemEntity> heritageItemEntityList) {
        return heritageItemEntityList.stream()
                .collect(Collectors.toMap(
                        HeritageItemEntity:: getId,
                        heritageItemEntity -> findTags(getTagsIds(heritageItemEntity.getTags()))
                ));
    }

    private List<Integer> getTagsIds(String tagsString) {
        if (tagsString == null || tagsString.isEmpty() || tagsString.equals("null")) return Collections.emptyList();

        return Arrays.stream(tagsString.split(","))
                .map(Integer::parseInt)
                .toList();
    }

    private List<FieldTagsEntity> findTags(List<Integer> tagsIds) {
        return entityManager
                .createNamedQuery(FieldTagsEntity.FIND_TAGS_BY_IDS, FieldTagsEntity.class)
                .setParameter("tagsIds", tagsIds)
                .getResultList();
    }

    private Map<Integer, List<HeritageListLinksEntity>> findHeritageListLinksByHeritageListId(List<HeritageItemEntity> heritageItemEntityList) {
        List<Integer> heritageListIds = heritageItemEntityList.stream()
                .map(HeritageItemEntity::getId)
                .toList();

        List<HeritageListLinksEntity> heritageListLinkEntityList = entityManager
                .createNamedQuery(HeritageListLinksEntity.FIND_BY_HERITAGE_LIST_ID, HeritageListLinksEntity.class)
                .setParameter("heritageListId", heritageListIds)
                .getResultList();

        return heritageListLinkEntityList.stream()
                .collect(Collectors.groupingBy(HeritageListLinksEntity::getHeritageListId));
    }

    private List<Integer> mapCategories(Types types) {
        return types.categories().stream()
                .map(Type::getId)
                .toList();
    }
}
