package com.ocity.api.core.infrastructure.heritage.entity;

import com.ocity.api.core.application.heritage.dto.in.HeritageFormat;
import com.ocity.api.core.domain.heritage.About;
import com.ocity.api.core.domain.heritage.AboutLanguage;
import com.ocity.api.core.domain.heritage.AboutText;
import com.ocity.api.core.domain.heritage.AddressCountry;
import com.ocity.api.core.domain.heritage.AddressLocality;
import com.ocity.api.core.domain.heritage.Description;
import com.ocity.api.core.domain.heritage.DescriptionLanguage;
import com.ocity.api.core.domain.heritage.DescriptionText;
import com.ocity.api.core.domain.heritage.Extras;
import com.ocity.api.core.domain.heritage.FormatType;
import com.ocity.api.core.domain.heritage.HeritageItem;
import com.ocity.api.core.domain.heritage.Id;
import com.ocity.api.core.domain.heritage.ImageUrl;
import com.ocity.api.core.domain.heritage.Latitude;
import com.ocity.api.core.domain.heritage.Link;
import com.ocity.api.core.domain.heritage.Links;
import com.ocity.api.core.domain.heritage.Longitude;
import com.ocity.api.core.domain.heritage.Name;
import com.ocity.api.core.domain.heritage.Tag;
import com.ocity.api.core.domain.heritage.TouristType;
import com.ocity.api.core.domain.heritage.TouristTypeMapper;
import com.ocity.api.core.domain.heritage.Url;
import com.ocity.api.core.domain.heritage.UrlFriendly;
import com.ocity.api.core.domain.heritage.schemaorg.SchemaOrgType;
import com.ocity.api.core.domain.heritage.schemaorg.SchemaOrgTypeMapper;
import com.ocity.api.core.domain.heritage.une.UNEType;
import com.ocity.api.core.domain.heritage.une.UneTypeMapper;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

@Component
public class HeritageItemMapper {

    public HeritageItem map(HeritageItemEntity heritageItemEntity,
                            Map<Integer, List<HeritageListLinksEntity>> linksByHeritageListId,
                            Map<Integer, List<FieldTagsEntity>> tagsByHeritageListId,
                            HeritageFormat heritageFormat) {
        return new HeritageItem(
                new Id(heritageItemEntity.getId()),
                new Name(heritageItemEntity.getName()),
                new AddressCountry(heritageItemEntity.getCountryCode()),
                new AddressLocality(heritageItemEntity.getCityName()),
                new Latitude(heritageItemEntity.getLatitude()),
                new Longitude(heritageItemEntity.getLongitude()),
                mapDescription(heritageItemEntity),
                heritageItemEntity.getImage().isBlank() ? null :  new ImageUrl(heritageItemEntity.getImage()),
                mapType(heritageItemEntity, tagsByHeritageListId, heritageFormat),
                mapTouristType(heritageItemEntity, tagsByHeritageListId),
                mapLinks(heritageItemEntity, linksByHeritageListId),
                mapAbout(heritageItemEntity),
                mapExtras(heritageItemEntity, tagsByHeritageListId)
        );
    }

    private List<Description> mapDescription(HeritageItemEntity heritageItemEntity) {
        return List.of(
                new Description(
                        new DescriptionLanguage("es"),
                        heritageItemEntity.getShortLocalHeritageDescription().isBlank()
                                ? null
                                : new DescriptionText(heritageItemEntity.getShortLocalHeritageDescription())
                ),
                new Description(
                        new DescriptionLanguage("en"),
                        heritageItemEntity.getShortHeritageDescription().isBlank()
                                ? null
                                : new DescriptionText(heritageItemEntity.getShortHeritageDescription())
                )
        );
    }

    private Links mapLinks(HeritageItemEntity heritageItemEntity, Map<Integer, List<HeritageListLinksEntity>> linksByHeritageListId) {
        Optional<List<HeritageListLinksEntity>> mayBeLinksByHeritage = linksByHeritageListId != null && linksByHeritageListId.get(heritageItemEntity.getId()) != null
                ? Optional.of(linksByHeritageListId.get(heritageItemEntity.getId()))
                : Optional.empty();

        return new Links(
                new UrlFriendly("PENDING"),
                mayBeLinksByHeritage
                        .map(linksByHeritage -> linksByHeritage.stream()
                                .map(link -> new Link(new Url(link.getUrl()), new DescriptionText(link.getName())))
                                .toList())
                        .orElse(null)
        );
    }

    private List<About> mapAbout(HeritageItemEntity heritageItemEntity) {
        return List.of(
                new About(
                        new AboutLanguage("es"),
                        heritageItemEntity.getShortLocalHeritageDescription().isBlank()
                                ? null
                                : new AboutText(heritageItemEntity.getShortLocalHeritageDescription())
                ),
                new About(
                        new AboutLanguage("en"),
                        heritageItemEntity.getShortHeritageDescription().isBlank()
                                ? null
                                : new AboutText(heritageItemEntity.getShortHeritageDescription())
                )
        );
    }

    private List<FormatType> mapType(HeritageItemEntity heritageItemEntity, Map<Integer, List<FieldTagsEntity>> tagsByHeritageListId, HeritageFormat heritageFormat) {
        Optional<List<FieldTagsEntity>> mayBeTagsByHeritageListId = tagsByHeritageListId != null && tagsByHeritageListId.get(heritageItemEntity.getId()) != null
                ? Optional.of(tagsByHeritageListId.get(heritageItemEntity.getId()))
                : Optional.empty();

        return mayBeTagsByHeritageListId
                .map(tags -> tags.stream()
                        .map(tag -> Tag.fromId(tag.getId()))
                        .map(tag -> mapFormatType(heritageFormat, tag))
                        .filter(Objects::nonNull)
                        .distinct()
                        .collect(Collectors.collectingAndThen(Collectors.toList(), list -> {
                            if (Boolean.TRUE.equals(heritageItemEntity.getIsMuseum())) {
                                list.add(getMuseumType(heritageFormat));
                            }
                            return list;
                        })))
                .orElse(Collections.emptyList());
    }

    private FormatType mapFormatType(HeritageFormat heritageFormat, Tag tag) {
        if (HeritageFormat.UNE.equals(heritageFormat)) return UneTypeMapper.getUneType(tag);

        return SchemaOrgTypeMapper.getSchemaOrgType(tag);
    }

    private FormatType getMuseumType(HeritageFormat heritageFormat) {
        if (HeritageFormat.UNE.equals(heritageFormat)) return UNEType.MUSEUM;

        return SchemaOrgType.MUSEUM;
    }

    private List<TouristType> mapTouristType(HeritageItemEntity heritageItemEntity,Map<Integer, List<FieldTagsEntity>> tagsByHeritageListId) {
        Optional<List<FieldTagsEntity>> mayBeTagsByHeritageListId = tagsByHeritageListId != null && tagsByHeritageListId.get(heritageItemEntity.getId()) != null
                ? Optional.of(tagsByHeritageListId.get(heritageItemEntity.getId()))
                : Optional.empty();

        if (mayBeTagsByHeritageListId.isEmpty()) return Collections.emptyList();


        return mayBeTagsByHeritageListId
                .map(tags -> tags.stream()
                        .map(FieldTagsEntity::getId)
                        .map(Tag::fromId)
                        .map(TouristTypeMapper::getTouristTypes)
                        .filter(Objects::nonNull)
                        .flatMap(List::stream)
                        .distinct()
                        .toList())
                .orElse(Collections.emptyList());
    }

    private Extras mapExtras(HeritageItemEntity heritageItemEntity, Map<Integer, List<FieldTagsEntity>> tagsByHeritageListId) {
        Optional<List<FieldTagsEntity>> mayBeTagsByHeritageListId = tagsByHeritageListId != null && tagsByHeritageListId.get(heritageItemEntity.getId()) != null
                ? Optional.of(tagsByHeritageListId.get(heritageItemEntity.getId()))
                : Optional.empty();

            return mayBeTagsByHeritageListId
                    .map(tags -> tags.stream()
                            .map(FieldTagsEntity::getName)
                            .collect(Collectors.collectingAndThen(Collectors.toList(), Extras::new)))
                    .orElse(null);
    }
}
