package com.ocity.api.core.infrastructure.locations;

import com.ocity.api.core.domain.locations.City;
import com.ocity.api.core.domain.locations.CityId;
import com.ocity.api.core.domain.locations.Country;
import com.ocity.api.core.domain.locations.CountryId;
import com.ocity.api.core.domain.locations.LocationsRepository;
import com.ocity.api.core.domain.locations.Name;
import com.ocity.api.core.domain.locations.Region;
import com.ocity.api.core.domain.locations.RegionId;
import com.ocity.api.core.domain.locations.State;
import com.ocity.api.core.domain.locations.StateId;
import com.ocity.api.core.domain.locations.Subregion;
import com.ocity.api.core.domain.locations.SubregionId;
import com.ocity.api.core.infrastructure.locations.entity.CityEntity;
import com.ocity.api.core.infrastructure.locations.entity.CountryEntity;
import com.ocity.api.core.infrastructure.locations.entity.RegionEntity;
import com.ocity.api.core.infrastructure.locations.entity.StateEntity;
import com.ocity.api.core.infrastructure.locations.entity.SubregionEntity;
import jakarta.persistence.EntityManager;
import jakarta.persistence.PersistenceContext;
import org.springframework.stereotype.Repository;

import java.util.Collections;
import java.util.List;

@Repository
public class LocationsRepositoryAdapter implements LocationsRepository {

    @PersistenceContext
    EntityManager entityManager;

    @Override
    public List<Region> findAllRegions() {
        return entityManager
                .createNamedQuery(RegionEntity.FIND_ALL_REGIONS, RegionEntity.class)
                .getResultList()
                .stream()
                .map(this::mapRegion)
                .toList();
    }

    @Override
    public List<Subregion> findAllSubRegions() {
        return entityManager
                .createNamedQuery(SubregionEntity.FIND_ALL_SUBREGIONS, SubregionEntity.class)
                .getResultList()
                .stream()
                .map(this::mapSubRegion)
                .toList();
    }

    @Override
    public List<Subregion> findSubregionsByRegionId(RegionId regionId) {
        return entityManager
                .createNamedQuery(SubregionEntity.FIND_SUBREGIONS_BY_REGION_ID, SubregionEntity.class)
                .setParameter("regionId", regionId.value())
                .getResultList()
                .stream()
                .map(this::mapSubRegion)
                .toList();
    }

    @Override
    public List<Country> findAllCountries() {
        return entityManager
                .createNamedQuery(CountryEntity.FIND_ALL_COUNTRIES, CountryEntity.class)
                .getResultList()
                .stream()
                .map(this::mapCountry)
                .toList();
    }

    @Override
    public List<Country> findCountriesBySubregionId(SubregionId subregionId) {
        return subregionId.value()
                .map(value -> entityManager
                        .createNamedQuery(CountryEntity.FIND_COUNTRIES_BY_SUBREGION_ID, CountryEntity.class)
                        .setParameter("subregionId", value)
                        .getResultList()
                        .stream()
                        .map(this::mapCountry)
                        .toList())
                .orElse(Collections.emptyList());
    }

    @Override
    public List<Country> findCountriesByRegionId(RegionId regionId) {
        return entityManager
                .createNamedQuery(CountryEntity.FIND_COUNTRIES_BY_REGION_ID, CountryEntity.class)
                .setParameter("regionId", regionId.value())
                .getResultList()
                .stream()
                .map(this::mapCountry)
                .toList();
    }

    @Override
    public List<State> findStatesByCountryId(CountryId countryId) {
        return entityManager
                .createNamedQuery(StateEntity.FIND_STATES_BY_COUNTRY_ID, StateEntity.class)
                .setParameter("countryId", countryId.value())
                .getResultList()
                .stream()
                .map(this::mapState)
                .toList();
    }

    @Override
    public List<City> findCitiesByStateId(StateId stateId) {
        return entityManager
                .createNamedQuery(CityEntity.FIND_CITIES_BY_STATE_ID, CityEntity.class)
                .setParameter("stateId", stateId.value())
                .getResultList()
                .stream()
                .map(this::mapCity)
                .toList();
    }

    private Region mapRegion(RegionEntity regionEntity) {
        return new Region(
                new RegionId(regionEntity.getId()),
                new Name(regionEntity.getName())
        );
    }

    private Subregion mapSubRegion(SubregionEntity subregionEntity) {
        return new Subregion(
                new SubregionId(subregionEntity.getId()),
                new Name(subregionEntity.getName()),
                new RegionId(subregionEntity.getRegionId())
        );
    }

    private Country mapCountry(CountryEntity countryEntity) {
        return new Country(
                new CountryId(countryEntity.getId()),
                new Name(countryEntity.getName()),
                new SubregionId(countryEntity.getSubregionId())
        );
    }

    private State mapState(StateEntity stateEntity) {
        return new State(
                new StateId(stateEntity.getId()),
                new Name(stateEntity.getName())
        );
    }

    private City mapCity(CityEntity cityEntity) {
        return new City(
                new CityId(cityEntity.getId()),
                new Name(cityEntity.getName())
        );
    }
}
