document.addEventListener('DOMContentLoaded', () => {
    const elements = {
        userApiKey: document.getElementById('api-key'),
        regions: document.getElementById('regions'),
        subregions: document.getElementById('subregions'),
        countries: document.getElementById('countries'),
        states: document.getElementById('states'),
        cities: document.getElementById('cities'),
        countriesContainer: document.getElementById('countries-container'),
        statesContainer: document.getElementById('states-container'),
        citiesContainer: document.getElementById('cities-container'),
        searchButton: document.getElementById('search-button'),
        downloadButton: document.getElementById('download-button'),
        toolbarContainer: document.getElementById('toolbar-container'),
        searchResponseContainer: document.getElementById('search-response-container'),
        searchResponse: document.getElementById('search-response'),
        culturalCategoryCheckbox: document.getElementById('cultural-category'),
        naturalCategoryCheckbox: document.getElementById('natural-category'),
        mixedCategoryCheckbox: document.getElementById('mixed-category'),
        formatToggleSwitch: document.getElementById('toggle-switch'),
    };

    const ACTION = Object.freeze({
        SEARCH: 'SEARCH',
        DOWNLOAD: 'DOWNLOAD'
    });

    const DATA_FORMAT = Object.freeze({
        UNE: { id: 1, name: 'UNE' },
        SCHEMA_ORG: { id: 2, name: 'SCHEMA_ORG' }
    });

    const { regions, subregions, countries, states, cities, countriesContainer, statesContainer, citiesContainer, culturalCategoryCheckbox, naturalCategoryCheckbox, mixedCategoryCheckbox } = elements;
    const locationSelectors = [regions, subregions, countries, states, cities];
    const categoryCheckboxes = [culturalCategoryCheckbox, naturalCategoryCheckbox, mixedCategoryCheckbox];
    const formatToggleSelector = elements.formatToggleSwitch;
    const searchButton = elements.searchButton;
    const downloadButton = elements.downloadButton;

    fetch('toolbar.html')
        .then(response => response.text())
        .then(data => elements.toolbarContainer.innerHTML = data)

    initPage();

    function initPage() {
        initSelectors();
        initLocations();
        fetchUserData();
    }

    function initSelectors() {
        locationSelectors.forEach(selector => {
            cleanSelect(selector);
            selector.disabled = true;
            selector.addEventListener('change', checkSelectors)
        });
    }

    function initLocations() {
        fetchRegions();
        fetchSubregions();
        fetchCountries();
    }

    function fetchUserData() {
        fetch('/user')
            .then(handleResponse)
            .then(data => {
                const { userApiKey } = elements;
                userApiKey.textContent = data.apiKey;
            })
            .catch(error => {
                console.error('Failed to retrieve user data:', error);
            });
    }

    function fetchRegions(regionId) {
        fetch(regionId ? `regions/${regiondId}/countries` : '/regions')
            .then(handleResponse)
            .then(data => populateSelect(regions, data))
            .catch(error => console.error('Failed to retrieve regions:', error));
    }

    function fetchSubregions(regionId) {
        fetch(regionId ? `/regions/${regionId}/subregions` : '/subregions')
            .then(handleResponse)
            .then(data => {
                populateSelect(subregions, data);
                cacheSubregions(data);
            })
            .catch(error => console.error('Failed to retrieve subregions:', error));
    }

    function fetchCountriesFromRegionId(regionId) {
        fetch(regionId ? `/regions/${regionId}/countries` : '/countries')
            .then(handleResponse)
            .then(data => {
                populateSelect(countries, data);
                cacheCountries(data);
            })
            .catch(error => console.error('Failed to retrieve countries:', error));
    }

    function fetchCountries(subregionId) {
        fetch(subregionId ? `/subregions/${subregionId}/countries` : '/countries')
            .then(handleResponse)
            .then(data => {
                populateSelect(countries, data);
                cacheCountries(data);
            })
            .catch(error => console.error('Failed to retrieve countries:', error));
    }

    function fetchStates(countryId) {
        fetch(`/countries/${countryId}/states`)
            .then(handleResponse)
            .then(data => populateSelect(states, data))
            .catch(error => console.error('Failed to retrieve states:', error));
    }

    function fetchCities(stateId) {
        fetch(`/states/${stateId}/cities`)
            .then(handleResponse)
            .then(data => populateSelect(cities, data))
            .catch(error => console.error('Failed to retrieve cities:', error));
    }

    function cacheSubregions(subregions) {
        subregions.forEach(subregion => localStorage.setItem(`regionIdForSubregionId_${subregion.id}`, subregion.regionId));
    }

    function cacheCountries(countries) {
        countries.forEach(country => localStorage.setItem(`subregionIdForCountryId_${country.id}`, country.subregionId));
    }

    function getRegionIdFromSubregion(subregionId) {
        return localStorage.getItem(`regionIdForSubregionId_${subregionId}`);
    }

    function getSubregionIdFromCountry(countryId) {
        return localStorage.getItem(`subregionIdForCountryId_${countryId}`);
    }

    function cleanSelect(selector) {
        Array.from(selector.options).forEach(option => {
            if (!option.disabled) option.remove();
        });
    }

    function resetSelect(selector) {
        selector.value = selector.querySelector('option:disabled').value;
    }

    function resetLowerSelectors(currentSelector) {
        const currentIndex = locationSelectors.indexOf(currentSelector);
        const lowerSelectors = locationSelectors.slice(currentIndex + 1);
        lowerSelectors.forEach(resetSelect);
        lowerSelectors.forEach(cleanSelect);
        const subLowerSelectors = locationSelectors.slice(currentIndex + 2);
        subLowerSelectors.forEach(selector => {
            if (!isHidden(selector.parentNode) && selector !== countries) {
                hideElementWithTransition(selector.parentNode)
            }
        });
    }

    function isHidden(element) {
        return element.classList.contains('hidden');
    }

    function showElementWithTransition(element) {
        element.classList.remove('hidden');
        element.classList.add('fade-in');
        element.addEventListener('animationend', function(event) {
            if (event.animationName === 'fadeIn') {
                element.classList.remove('fade-in');
            }
        });
    }

    function hideElementWithTransition(element) {
        element.classList.add('fade-out');
        element.addEventListener('animationend', function(event) {
            if (event.animationName === 'fadeOut') {
                element.classList.add('hidden');
                element.classList.remove('fade-out');
            }
        });
    }

    function populateSelect(selector, data) {
        cleanSelect(selector);
        data.forEach(item => {
            const option = document.createElement('option');
            option.value = item.id;
            option.textContent = item.name;
            selector.appendChild(option);
        });
        enableSelector(selector);
    }

    function enableSelector(selector) {
        selector.disabled = false;
    }

    regions.addEventListener('change', handleRegionChange);
    subregions.addEventListener('change', handleSubregionChange);
    countries.addEventListener('change', handleCountryChange);
    states.addEventListener('change', handleStateChange);
    searchButton.addEventListener('click', () => doAction(ACTION.SEARCH));
    downloadButton.addEventListener('click', () => doAction(ACTION.DOWNLOAD));
    categoryCheckboxes.forEach(() => addEventListener('change', checkIfSearchButtonShouldBeEnabled));

    function handleRegionChange() {
        const regionId = regions.value;
        [subregions, countries].forEach(selector => selector.disabled = true);
        if (regionId) {
            resetLowerSelectors(regions);
            fetchSubregions(regionId);
            fetchCountriesFromRegionId(regionId);
            checkSelectors();
        }
    }

    function handleSubregionChange() {
        const subregionId = subregions.value;
        const regionId = getRegionIdFromSubregion(subregionId);
        if (subregionId) {
            resetLowerSelectors(subregions);
            fetchCountries(subregionId, regionId);
            regions.value = regionId;
            showElementWithTransition(countriesContainer);
            checkSelectors();
        }
    }

    function handleCountryChange() {
        const countryId = countries.value;
        const subregionId = getSubregionIdFromCountry(countryId);
        if (countryId) {
            resetLowerSelectors(countries);
            fetchStates(countryId);
            subregions.value = subregionId;
            regions.value = getRegionIdFromSubregion(subregionId);
            showElementWithTransition(statesContainer);
            checkSelectors();
        }
    }

    function handleStateChange() {
        const stateId = states.value;
        if (stateId) {
            resetLowerSelectors(states)
            fetchCities(stateId);
            showElementWithTransition(citiesContainer);
            checkSelectors();
        }
    }

    function doAction(action) {
        const lowerSelectedSelector = findLowestLocationSelectedSelector();
        const checked = getMarkedCheckboxes();
        let queryParams = checked.map(item => `${encodeURIComponent(item)}=true`).join('&');

        const formatId = formatToggleSelector.checked
            ? DATA_FORMAT.SCHEMA_ORG.id
            : DATA_FORMAT.UNE.id;

        queryParams = queryParams.concat('&').concat('data-format=').concat(formatId);

        let url;

        const buildUrl = (baseUrl) => queryParams ? `${baseUrl}?${queryParams}` : baseUrl;

        const actionUrl = ACTION.SEARCH === action
            ? 'data'
            : 'download';

        switch (lowerSelectedSelector) {
            case regions:
                url = buildUrl(`/regions/${regions.value}/${actionUrl}`);
                break;
            case subregions:
                url = buildUrl(`/subregions/${subregions.value}/${actionUrl}`);
                break;
            case countries:
                url = buildUrl(`/countries/${countries.value}/${actionUrl}`);
                break;
            case states:
                url = buildUrl(`/states/${states.value}/${actionUrl}`);
                break;
            case cities:
                url = buildUrl(`/cities/${cities.value}/${actionUrl}`);
                break;
        }
        elements.searchResponse.value = "";
        switch (action) {
            case ACTION.SEARCH:
                showToast(`Retrieving heritage data`,"info",5000);
                fetch(url)
                    .then(handleResponse)
                    .then(data => {
                        showElementWithTransition(elements.searchResponseContainer);
                        elements.searchResponse.value = JSON.stringify(data, null, 2)
                        elements.searchResponse.style.height = 'auto';
                        elements.searchResponse.style.height = (elements.searchResponse.scrollHeight) + 'px';
                        showToast(`Retrieved data succesfully`,"success",5000);
                    })
                    .catch(error => {
                        console.error('Failed to retrieve data:', error);
                        showToast(`Failed to retrieve data:\n${error}`,"danger",5000);
                    });
                break;
            case ACTION.DOWNLOAD:
                elements.searchResponseContainer.classList.add('hidden');
                showToast(`Retrieving heritage data`,"info",5000);
                fetch(url)
                    .then(handleDownloadResponse)
                    .then(({buffer, filename}) => {
                        const blob = new Blob([buffer], { type: 'application/octet-stream'});
                        const tempUrl = window.URL.createObjectURL(blob);
                        const a = document.createElement('a');
                        a.style.display = 'none';
                        a.href = tempUrl;
                        a.download = filename;
                        document.body.appendChild(a);
                        a.click();
                        window.URL.revokeObjectURL(tempUrl);
                        a.remove();
                        showToast(`Retrieved data succesfully`,"success",5000);
                    })
                    .catch(error => {
                        console.error('Failed to retrieve data:', error);
                        showToast(`Failed to retrieve data:\n${error}`,"danger",5000);
                    });
                break;
        }
    }

    function handleResponse(response) {
        if (response.ok) {
            return response.json()
        }
        if (response.status === 500) {
            console.error('Internal server error');
            throw new Error('Internal server error');
        }
        if (response.status === 401) {
            console.error('Unauthorized');
            window.location.replace('index.html');
        }
        throw new Error('Failed to retrieve data');
    }

    function handleDownloadResponse(response) {
        if (response.ok) {
            const contentDisposition = response.headers.get('Content-Disposition');
            let filename = 'data.json';
            if (contentDisposition) {
                const filenameRegex = /filename[^;=\n]*=(['"])(.*?)\1/;
                const matches = filenameRegex.exec(contentDisposition);
                if (matches != null && matches[2]) {
                    filename = matches[2];
                }
            }

            return response.arrayBuffer().then(buffer => {
                return { buffer, filename }
            });
        }

        if (response.status === 500) {
            console.error('Internal server error');
            throw new Error('Internal server error');
        }
        if (response.status === 401) {
            console.error('Unauthorized');
            window.location.replace('index.html');
        }
        throw new Error('Failed to retrieve data');
    }

    function checkSelectors() {
        let lastNonDefaultSelector = findLowestLocationSelectedSelector();

        locationSelectors.forEach(selector => {
            selector.classList.remove('highlighted');

            selector.selectedIndex === 0
                ? selector.classList.add('default-selection')
                : selector.classList.remove('default-selection');

        });

        if (lastNonDefaultSelector) lastNonDefaultSelector.classList.add('highlighted');

        checkIfSearchButtonShouldBeEnabled();
    }

    function checkIfSearchButtonShouldBeEnabled() {
        const anyLocationSelectorSelected = locationSelectors.some(selector =>
            selector.value !== '' &&
            selector.value !== selector.querySelector('option:disabled').value
        );

        const anyCategoryFilterSelected = categoryCheckboxes.some(checkbox =>
            checkbox.checked
        );

        searchButton.disabled = !(anyLocationSelectorSelected && anyCategoryFilterSelected);
        downloadButton.disabled = !(anyLocationSelectorSelected && anyCategoryFilterSelected);
    }

    function findLowestLocationSelectedSelector() {
        let lastNonDefaultSelector;
        locationSelectors.forEach(selector => {
            if (selector.selectedIndex !== 0) {
                lastNonDefaultSelector = selector;
            }
        });
        return lastNonDefaultSelector;
    }

    function getMarkedCheckboxes() {
        let selectedCheckboxes = [];
        categoryCheckboxes.forEach(checkbox => {
            if (checkbox.checked) {
                selectedCheckboxes.push(checkbox.id);
            }
        });
        return selectedCheckboxes;
    }

    let icon = {
        success: '<span class="material-symbols-outlined">task_alt</span>',
        danger: '<span class="material-symbols-outlined">error</span>',
        info: '<span class="material-symbols-outlined">info</span>',
    };

    const showToast = (
        message = "Sample Message",
        toastType = "info",
        duration = 5000) => {
        if (
            !Object.keys(icon).includes(toastType))
            toastType = "info";

        let box = document.createElement("div");
        box.classList.add(
            "toast", `toast-${toastType}`);
        box.innerHTML = ` <div class="toast-content-wrapper"> 
                      <div class="toast-icon"> 
                      ${icon[toastType]} 
                      </div> 
                      <div class="toast-message">${message}</div> 
                      <div class="toast-progress"></div> 
                      </div>`;
        duration = duration || 5000;
        box.querySelector(".toast-progress").style.animationDuration =
            `${duration / 1000}s`;

        let toastAlready =
            document.body.querySelector(".toast");
        if (toastAlready) {
            toastAlready.remove();
        }

        document.body.appendChild(box)};
});
